/**
 * Copyright 2011 Marco Buzzanca. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, are
 * permitted provided that the following conditions are met:
 * 
 *    1. Redistributions of source code must retain the above copyright notice, this list of
 *       conditions and the following disclaimer.
 * 
 *    2. Redistributions in binary form must reproduce the above copyright notice, this list
 *       of conditions and the following disclaimer in the documentation and/or other materials
 *       provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER ''AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * The views and conclusions contained in the software and documentation are those of the
 * authors and should not be interpreted as representing official policies, either expressed
 * or implied, of the copyright holder.
 */
 
package shx.math;

public class Complex
{
	private final double re;
	private final double im;
	
	public Complex(double re, double im)
	{
		this.re = re;
		this.im = im;
	}
	
	public Complex(Complex c)
	{
		this.re = c.re;
		this.im = c.im;
	}
	
	public double re() { return re; }
	public double im() { return im; }
	
	public Complex add(Complex c)
	{
		return new Complex(re + c.re, im + c.im);
	}
	
	public Complex sub(Complex c)
	{
		return new Complex(re - c.re, im - c.im);
	}
	
	public Complex mul(Complex c)
	{
		return new Complex(re * c.re - im * c.im, re * c.im + c.re * im);
	}
	
	public Complex mul(double k)
	{
		return new Complex(re * k, im * k);
	}
	
	public Complex div(Complex c)
	{
		double den = c.re * c.re + c.im * c.im;
		double nRe = (re * c.re + im * c.im) / den;
		double nIm = (im * c.re - re * c.im) / den;
		
		return new Complex(nRe, nIm);
	}
	
	public Complex div(double k)
	{
		return new Complex(re / k, im / k);
	}
	
	
	public Complex divs(Complex c)
	{
		double den = c.re * c.re + c.im * c.im;
		
		if (den == 0d)
			den = Double.MIN_VALUE;
			
		double nRe = (re * c.re + im * c.im) / den;
		double nIm = (im * c.re - re * c.im) / den;
		
		return new Complex(nRe, nIm);
	}
	
	public Complex divs(double k)
	{
		if (k == 0d)
			return new Complex(re / Double.MIN_VALUE, im / Double.MIN_VALUE);
		
		return new Complex(re / k, im / k);
	}
	
	public Complex conj()
	{
		return new Complex(re, -im);
	}

}
