/**
 * Copyright 2011 Marco Buzzanca. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, are
 * permitted provided that the following conditions are met:
 * 
 *    1. Redistributions of source code must retain the above copyright notice, this list of
 *       conditions and the following disclaimer.
 * 
 *    2. Redistributions in binary form must reproduce the above copyright notice, this list
 *       of conditions and the following disclaimer in the documentation and/or other materials
 *       provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER ''AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * The views and conclusions contained in the software and documentation are those of the
 * authors and should not be interpreted as representing official policies, either expressed
 * or implied, of the copyright holder.
 */

package shx.security;

import java.io.IOException;
import java.io.FileInputStream;
import java.io.InputStream;
import java.security.MessageDigest;
import shx.utils.StringUtils;

public class FileDigest
{
	private MessageDigest hashAlgorithm;
	
	public void setAlgorithm(String algorithm) 
	{ 
		try { hashAlgorithm = MessageDigest.getInstance(algorithm); }
		catch (Exception e) { hashAlgorithm = null; }
	}
	
	public String getAlgorithm() 
	{ 
		return hashAlgorithm.getAlgorithm(); 
	}
	
	public FileDigest(String algorithm) 
	{ 
		try { hashAlgorithm = MessageDigest.getInstance(algorithm); }
		catch (Exception e) { hashAlgorithm = null; }
	}
	
	public byte[] digest(String filename) 
		throws IOException
	{
		InputStream in = new FileInputStream(filename);
		
		byte[] buffer = new byte[1024];
		int read;
		
		do 
		{
			read = in.read(buffer);
			
			if (read > 0)
				hashAlgorithm.update(buffer, 0, read);

		} while (read != -1);
		
		in.close();
		
		return hashAlgorithm.digest();
	}
	
	public String hash(String filename) 
		throws IOException
	{
		return StringUtils.getHexString(digest(filename));
	}
	
	public static byte[] digest(String algorithm, String filename) 
		throws IOException
	{
		return (new FileDigest(algorithm)).digest(filename);
	}
	
	public static String hash(String algorithm, String filename)
		throws IOException
	{
		byte[] digest = FileDigest.digest(algorithm, filename);
		return StringUtils.getHexString(digest);
	}
	

}
