/**
 * Copyright 2011 Marco Buzzanca. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, are
 * permitted provided that the following conditions are met:
 * 
 *    1. Redistributions of source code must retain the above copyright notice, this list of
 *       conditions and the following disclaimer.
 * 
 *    2. Redistributions in binary form must reproduce the above copyright notice, this list
 *       of conditions and the following disclaimer in the documentation and/or other materials
 *       provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER ''AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * The views and conclusions contained in the software and documentation are those of the
 * authors and should not be interpreted as representing official policies, either expressed
 * or implied, of the copyright holder.
 */
 
package shx.web;

import java.util.*;
import java.util.regex.*;
import java.io.*;

public class HTMLTemplate
{
	private Map<String, String> dictionary;
	private String html;
	
	private boolean loaded;
	private boolean parsed;
	
	public HTMLTemplate()
	{ 
		dictionary = new HashMap<String, String>(); 
		loaded = parsed = false;
	}
	
	public HTMLTemplate(Map<String, String> dictionary) 
	{ 
		this.dictionary = dictionary; 
		loaded = parsed = false;
	}
	
	public String getHTML()
	{
		if (parsed) return html;
		return null;
	}
	
	public String addToken(String token, String value)
	{
		return dictionary.put(token, value);
	}
	
	public String getTokenValue(String token) 
	{
		return dictionary.get(token);
	}
	
	public String removeToken(String token)
	{
		return dictionary.remove(token);
	}
	
	public void wipeTokens()
	{
		dictionary = new HashMap<String, String>(); 
	}
	
	// Loads template file content
	public boolean load(String templateFilename)
	{
		try
		{
			BufferedInputStream in = new BufferedInputStream(new FileInputStream(templateFilename));
			byte[] buffer = new byte[1024];

			html = "";
			int read = 0;

			while ((read = in.read(buffer)) != -1)
				html += new String(buffer, 0, read);
			
			in.close();
		
			loaded = true;
			parsed = false;
		}
		
		catch (Exception e) { return false; }
		
		return true;
	}
	
	// It can be useful if HTMLTemplate is used in a .jar file
	public boolean load(InputStream templateStream)
	{
		try
		{
			BufferedInputStream in = new BufferedInputStream(templateStream);
			byte[] buffer = new byte[1024];

			html = "";
			int read = 0;

			while ((read = in.read(buffer)) != -1)
				html += new String(buffer, 0, read);
			
			in.close();
		
			loaded = true;
			parsed = false;
		}
		
		catch (Exception e) { return false; }
		
		return true;
	}
	
	// Replaces tokens with their actual value
	public boolean parse()
	{
		try
		{
			if (loaded)
			{
				for (Map.Entry<String, String> entry : dictionary.entrySet())
					html = html.replaceAll("\\{" + Pattern.quote(entry.getKey()) + "\\}", entry.getValue());
				
				parsed = true;
			}
		}
		
		catch (Exception e) { return false; }
		
		return true;
	}
	
	// Writes HTML file
	public boolean save(String path)
	{
		try
		{
			if (parsed)
			{
				BufferedWriter out = new BufferedWriter(new FileWriter(path));
				out.write(html);
				out.close();
			}
		}
		
		catch (Exception e) { return false; }
		
		return true;
	}
	
}
